//*****************************************************************************
// Mood Lamp
// Version 2.0 May 2009
//
// Sylvain Bissonnette
//*****************************************************************************
//
//            					R E T U R N   S T A C K   3 2
//                   					X T A L  16 MHZ
//
//*****************************************************************************
//                P I N   U S A G E
//
// PB0 -> IR input
// PB1 -> n/c
// PB2 -> n/c
// PB3 -> n/c
// PB4 -> n/c
// PB5 -> n/c
// PB6 -> Xtal
// PB7 -> Xtal
//
// PC0 -> LED 3 Red  
// PC1 -> LED 3 Green
// PC2 -> LED 3 Blue
// PC3 -> n/c
// PC4 -> n/c
// PC5 -> n/c
// PC6 -> RESET
//
// PD0 -> LED 1 Red
// PD1 -> LED 1 Green
// PD2 -> LED 1 Blue
// PD3 -> LED 2 Red
// PD4 -> LED 2 Green
// PD5 -> LED 2 Blue
// PD6 -> n/c
// PD7 -> n/c
//
// ADC6-> n/c
// ADC7-> n/c
//
//*****************************************************************************
//                T I M E R   U S A G E
//
// Timer 0 is not used
// Timer 1 is use by IR remote controle
// Timer 2 is use by Led PWM
//
//*****************************************************************************

//*****************************************************************************
//                      I N C L U D E
//*****************************************************************************
#include <iom324v.h>
#include <macros.h>

//*****************************************************************************
//                      D E F I N E
//*****************************************************************************
#define TRUE                  1
#define FALSE                 0
#define XTAL                  16000000
#define ushort				  unsigned short
#define uint				  unsigned int
#define ulong				  unsigned long

#define LED1_RED		      0x01 //(1<<PD0)
#define LED1_GREEN            (1<<PD1)
#define LED1_BLUE	    	  (1<<PD2)
#define LED1_DDR		      DDRD
#define LED1_PORT	    	  PORTD
#define LED1_PIN		      PIND

#define LED2_RED		      (1<<PD3)
#define LED2_GREEN            (1<<PD4)
#define LED2_BLUE	    	  (1<<PD5)
#define LED2_DDR		      DDRD
#define LED2_PORT	    	  PORTD
#define LED2_PIN		      PIND

#define LED3_RED		      (1<<PC0)
#define LED3_GREEN             (1<<PC1)
#define LED3_BLUE	    	  (1<<PC2)
#define LED3_DDR		      DDRC
#define LED3_PORT	    	  PORTC
#define LED3_PIN		      PINC

#define OFF					  0x00
#define RED					  0x01
#define GREEN				  0x02
#define BLUE				  0x03

#define IR_DDR                DDRD
#define IR_PIN                PIND
#define IR_PORT               PORTD
#define IR_INPUT              0x40

#define REMOTE_UP             20
#define REMOTE_DOWN           21
#define REMOTE_RIGHT          22
#define REMOTE_LEFT           23
#define REMOTE_MENU           24
#define REMOTE_STOP           30
#define REMOTE_PLAY           31
#define REMOTE_MUTE           32
#define REMOTE_REW            33
#define REMOTE_FFWD           34
#define REMOTE_PAUSE          35
#define REMOTE_POWER          40
#define REMOTE_NO_KEY		  -1

//*****************************************************************************
//                  P R O T O T Y P E
//*****************************************************************************
void main(void);

ushort GetRand(ushort Mask);
ushort Delay(ulong DelayStep, ulong DelayMin);

void RGBAllDC(ulong Delay, ulong DelayStep, ulong DelayMin);
void ClearAll(void);
void RandomAll(ulong Delay, ulong DelayStep, ulong DelayMin);
void RandomEqualAll(ulong Delay, ulong DelayStep, ulong DelayMin);
void RandomSmoothAll(ulong Delay, ulong DelayStep, ulong DelayMin);
void RGBAll(ulong Delay, ulong DelayStep, ulong DelayMin);
void ColorMixAll(ulong Delay, ulong DelayStep, ulong DelayMin);
void ColorMix(ulong Delay, ulong DelayStep, ulong DelayMin);
void Sablier(ulong Delay, ulong DelayStep, ulong DelayMin);
void Chaser1(ulong Delay, ulong DelayStep, ulong DelayMin);
void Chaser2(ulong Delay, ulong DelayStep, ulong DelayMin);
void Chaser3(ulong Delay, ulong DelayStep, ulong DelayMin);
void Chaser4(ulong Delay, ulong DelayStep, ulong DelayMin);

void LedSetup(ushort Led,ushort Color);

void ADInit(void);

void LedInit(void);
void LedStartShow(void);
void LedEngine(void);

void IRInit(void);
void IRLightChange(void);
void IRDecode(void);

//*****************************************************************************
//                    G L O B A L   V A R I A B L E
//*****************************************************************************
uint LatchedIrData = 0;
uint RemoteKey = 1;

ushort Led1Red;
ushort Led1Green;
ushort Led1Blue;

ushort Led2Red;
ushort Led2Green;
ushort Led2Blue;

ushort Led3Red;
ushort Led3Green;
ushort Led3Blue;

ulong VariableDelay;

uint Menu;
ushort Brightness;

//*****************************************************************************
//                            M A I N
//*****************************************************************************
void main()
{
ulong i;
ushort First, Second;

WDR();
WDTCR = 0x0f;     // Enable WatchDog at 2.2 sec

IRInit();
LedInit();
ADInit();
SEI();

Brightness = 64;
Menu = 6;

while(TRUE)
{
		WDR();
		if      (Menu == 1)  RandomAll(30000,1000,2000);
		else if (Menu == 2)  RandomEqualAll(200,50,50);
		else if (Menu == 3)  RandomSmoothAll(400,10,10);
		else if (Menu == 4)  RGBAll(400,50,50);
		else if (Menu == 5)  ColorMixAll(300,25,25);
		else if (Menu == 6)  ColorMix(300,25,25);
		else if (Menu == 7)  Sablier(300,25,25);
		else if (Menu == 8)  RGBAllDC(20000,500,2000);
		else if (Menu == 9)  Chaser1(10000, 500, 1000);
		else if (Menu == 10) Chaser2(10000, 500, 1000);
		else if (Menu == 11) Chaser3(10000, 500, 1000);
		else if (Menu == 12) Chaser4(10000, 500, 1000);

		ClearAll();
		for (i=0;i<50000;i++) WDR();
        
 		while(TRUE)
		{
		    for (i=0;i<10000;i++) WDR();
			
		    if (RemoteKey != REMOTE_NO_KEY)
			{
			   if (RemoteKey == REMOTE_MENU) 
			   {   
			   	   for (i=0;i<50000;i++) WDR();
			       RemoteKey = REMOTE_NO_KEY;
			       break;
			   }   
			   else
			   {
		 	       Second = First;
			   	   First = RemoteKey;
			   	   Menu = (Second * 10) + First;
			   	   for (i=0;i<50000;i++) WDR();
			   	   RemoteKey = REMOTE_NO_KEY;
			   } 
			}
		}
	}
}

/******************************************************************************

Name:         void ClearAll(void)

Description:

Input:        none

Output:       none

Misc:					
******************************************************************************/
void ClearAll(void)
{
	Led1Red =   0;
	Led1Green = 0;
	Led1Blue =  0;
	
	Led2Red = 	0;
	Led2Green = 0;
	Led2Blue =  0;
	
	Led3Red = 	0;
	Led3Green = 0;
	Led3Blue =  0;
}

/******************************************************************************

name:         void randomall(void)

description:

input:        none

output:       none

misc:					#1  
******************************************************************************/
void RandomAll(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{
    VariableDelay = DelayTick;

	while(TRUE)
	{
		WDR();
		
		Led1Red =   GetRand(Brightness);
		Led1Green = GetRand(Brightness);
		Led1Blue =  GetRand(Brightness);
				
		Led2Red =   Led1Red;
		Led2Green = Led1Green;
		Led2Blue =  Led1Blue;
		
		Led3Red =   Led1Red;
		Led3Green = Led1Green;
		Led3Blue =  Led1Blue;		

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
	}
	return;
}

/******************************************************************************

Name:         void RandomEqualAll(void)

Description:

Input:        none

Output:       none

Misc:					#2
******************************************************************************/
void RandomEqualAll(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{
	ushort i;
	ushort First = TRUE;

	VariableDelay = DelayTick;
    
	while(TRUE)
	{
		WDR();

		if (First)
		{
			First = FALSE;
			for (i=0;i<Brightness;i++)
			{
				if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
				
				Led1Red = i;
				Led2Red = i;
				Led3Red = i;
			}
		}

		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			Led1Green = i;
			Led2Green = i;
			Led3Green = i;
		}

		for (i=Brightness;i!=0;i--)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 

			Led1Red = i;
			Led2Red = i;
			Led3Red = i;
		}

		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			Led1Blue = i;
			Led2Blue = i;
			Led3Blue = i;
		}

		for (i=Brightness;i!=0;i--)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			Led1Green = i;
			Led2Green = i;
			Led3Green = i;
		}

		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 

			Led1Red = i;
			Led2Red = i;
			Led3Red = i;
		}

		for (i=Brightness;i!=0;i--)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			Led1Blue = i;
			Led2Blue = i;
			Led3Blue = i;
		}
	}
	return;
}

/******************************************************************************

Name:         void RandomSmoothAll(void)

Description:  

Input:        none

Output:       none

Misc:					#3
******************************************************************************/
void RandomSmoothAll(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{
	ushort RndRed = 0x7f;
	ushort RndGreen = 0x30;
	ushort RndBlue = 0x90;

	VariableDelay = DelayTick;

	while(TRUE)
	{
		WDR();

		if (Led1Red > RndRed) Led1Red--;
		if (Led1Red < RndRed) Led1Red++;
		if (Led1Red == RndRed) RndRed = GetRand(Brightness);

		if (Led1Green > RndGreen) Led1Green--;
		if (Led1Green < RndGreen) Led1Green++;
		if (Led1Green == RndGreen) RndGreen = GetRand(Brightness);

		if (Led1Blue > RndBlue) Led1Blue--;
		if (Led1Blue < RndBlue) Led1Blue++;
		if (Led1Blue == RndBlue) RndBlue = GetRand(Brightness);

		Led2Red = 	Led1Red;
		Led2Green = Led1Green;
		Led2Blue =  Led1Blue;
		
		Led3Red = 	Led1Red;
		Led3Green = Led1Green;
		Led3Blue =  Led1Blue;		

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
	}
	return;
}

/******************************************************************************

Name:         void RGBAll(void)

Description:

Input:        none

Output:       none

Misc:					#4
******************************************************************************/
void RGBAll(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{
	short RedAdd = 1;
	short GreenAdd = 1;
	short BlueAdd = 1;

	VariableDelay = DelayTick;

	while(TRUE)
	{
		WDR();
		
		Led1Red += RedAdd;
		if (Led1Red > Brightness) RedAdd = -1;
		if (Led1Red == 0x00) RedAdd = 0;

		if (RedAdd == 0)
		{
			Led1Green += GreenAdd;
			if (Led1Green > Brightness) GreenAdd = -1;
			if (Led1Green == 0x00) GreenAdd = 0;
		}

		if (GreenAdd == 0)
		{
			Led1Blue += BlueAdd;
			if (Led1Blue > Brightness) BlueAdd = -1;
			if (Led1Blue == 0x00) BlueAdd = 0;
		}

		if ((RedAdd == 0) && (GreenAdd == 0) && (BlueAdd == 0))
		{
			RedAdd = 1;
			GreenAdd = 1;
			BlueAdd = 1;
		}

		Led2Red =   Led1Red;
		Led2Green = Led1Green;
		Led2Blue =  Led1Blue;

		Led3Red = 	Led1Red;
		Led3Green = Led1Green;
		Led3Blue =  Led1Blue;		
		
		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
	}
}

/******************************************************************************

Name:         void ColorMixAll(void)

Description:

Input:        none

Output:       none

Misc:					#5
******************************************************************************/
void ColorMixAll(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{
	short RedAdd = 0;
	short GreenAdd = 0;
	short BlueAdd = 0;
	ushort i = 0;

	VariableDelay = DelayTick;

	while(TRUE)
	{
		WDR();

		i = GetRand(0x07);

		if ((i & 0x01) == 0x01)
		{
			Led1Red += RedAdd;
			if (Led1Red > Brightness) RedAdd = -1;
			if (Led1Red == 0x00) RedAdd = 1;
		}

		if ((i & 0x02) == 0x02)
		{
			Led1Green += GreenAdd;
			if (Led1Green  > Brightness) GreenAdd = -1;
			if (Led1Green == 0x00) GreenAdd = 1;
		}

		if ((i & 0x04) == 0x04)
		{
			Led1Blue += BlueAdd;
			if (Led1Blue > Brightness) BlueAdd = -1;
			if (Led1Blue == 0x00) BlueAdd = 1;
		}

		if (i == 0x07) i = 1;

		Led2Red =   Led1Red;
		Led2Green = Led1Green;
		Led2Blue =  Led1Blue;
		
		Led3Red = 	Led1Red;
		Led3Green = Led1Green;
		Led3Blue =  Led1Blue;				

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 
	}
	return;
}

/******************************************************************************

Name:         void ColorMix(void)

Description:

Input:        none

Output:       none

Misc:					#6
******************************************************************************/
void ColorMix(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{

	ushort RndRed1 = 0x7f;
	ushort RndGreen1 = 0x30;
	ushort RndBlue1 = 0x90;
	
	ushort RndRed2 = 0x10;
	ushort RndGreen2 = 0x30;
	ushort RndBlue2 = 0xe0;	
	
	ushort RndRed3 = 0xf0;
	ushort RndGreen3 = 0x03;
	ushort RndBlue3 = 0x70;		

	VariableDelay = DelayTick;

	while(TRUE)
	{
		WDR();

		if (Led1Red > RndRed1) Led1Red--;
		if (Led1Red < RndRed1) Led1Red++;
		if (Led1Red == RndRed1) RndRed1 = GetRand(Brightness);

		if (Led1Green > RndGreen1) Led1Green--;
		if (Led1Green < RndGreen1) Led1Green++;
		if (Led1Green == RndGreen1) RndGreen1 = GetRand(Brightness);

		if (Led1Blue > RndBlue1) Led1Blue--;
		if (Led1Blue < RndBlue1) Led1Blue++;
		if (Led1Blue == RndBlue1) RndBlue1 = GetRand(Brightness);
		
		//---
		
		if (Led2Red > RndRed2) Led2Red--;
		if (Led2Red < RndRed2) Led2Red++;
		if (Led2Red == RndRed2) RndRed2 = GetRand(Brightness);

		if (Led2Green > RndGreen2) Led2Green--;
		if (Led2Green < RndGreen2) Led2Green++;
		if (Led2Green == RndGreen2) RndGreen2 = GetRand(Brightness);

		if (Led2Blue > RndBlue2) Led2Blue--;
		if (Led2Blue < RndBlue2) Led2Blue++;
		if (Led2Blue == RndBlue2) RndBlue2 = GetRand(Brightness);		

		//---
		
		if (Led3Red > RndRed3) Led3Red--;
		if (Led3Red < RndRed3) Led3Red++;
		if (Led3Red == RndRed3) RndRed3 = GetRand(Brightness);

		if (Led3Green > RndGreen3) Led3Green--;
		if (Led3Green < RndGreen3) Led3Green++;
		if (Led3Green == RndGreen3) RndGreen3 = GetRand(Brightness);

		if (Led3Blue > RndBlue3) Led3Blue--;
		if (Led3Blue < RndBlue3) Led3Blue++;
		if (Led3Blue == RndBlue3) RndBlue3 = GetRand(Brightness);		

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
	}
	return;
}

/******************************************************************************

Name:         void Sablier(ulong Delay, ulong DelayStep, ulong DelayMin)

Description:

Input:        none

Output:       none

Misc:					#7
******************************************************************************/
void Sablier(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{
	ushort i;
	ushort First = TRUE;

	VariableDelay = DelayTick;

	while(TRUE)
	{
		WDR();
		
		if (First)
		{
			for (i=0;i<Brightness;i++)
			{
				if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

				Led1Red = i;
			}
			First = FALSE;
		}

		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
			
			Led1Red = Brightness - i;
			Led2Red = i;
		}

		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			Led2Red = Brightness - i;
			Led3Red = i;
		}

		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 

			Led3Red = Brightness - i;
			Led1Green = i;
		}


		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 

			Led1Green = Brightness - i;
			Led2Green = i;

		}

		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			Led2Green = Brightness - i;
			Led3Green =i;
		}

		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			Led3Green = Brightness - i;
			Led1Blue = i;
		}

		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			Led1Blue = Brightness - i;
			Led2Blue= i;
		}

		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			Led2Blue = Brightness - i;
			Led3Blue = i;
		}

		for (i=0;i<Brightness;i++)
		{
			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			Led3Blue = Brightness - i;
			Led1Red =  i;
		}
	}
	return;
}

/******************************************************************************

Name:         void RGBAllDC(void)

Description:

Input:        none

Output:       none

Misc:					#8  
******************************************************************************/
void RGBAllDC(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{
	VariableDelay = DelayTick;
	
	while(TRUE)
	{
		WDR();
		
		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
		
		LedSetup(1,RED);
		LedSetup(2,RED);
		LedSetup(3,RED);
		
		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 
		
		LedSetup(1,GREEN);
		LedSetup(2,GREEN);
		LedSetup(3,GREEN);
		
		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
		
		LedSetup(1,BLUE);
		LedSetup(2,BLUE);
		LedSetup(3,BLUE);				
	}
	return;
}

/******************************************************************************

Name:         void Chaser1(void)

Description:

Input:        none

Output:       none

Misc:					#9
******************************************************************************/
void Chaser1(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{
 	VariableDelay = DelayTick;

	while(TRUE)
	{
		WDR();
		
		LedSetup(1,GREEN);
		LedSetup(2,RED);
		LedSetup(3,RED);

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 

		LedSetup(1,GREEN);
		LedSetup(2,GREEN);
		LedSetup(3,RED);

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 

		LedSetup(1,GREEN);
		LedSetup(2,GREEN);
		LedSetup(3,GREEN);

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

		LedSetup(1,BLUE);
		LedSetup(2,GREEN);
		LedSetup(3,GREEN);

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 

		LedSetup(1,BLUE);
		LedSetup(2,BLUE);
		LedSetup(3,GREEN);

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
	
		LedSetup(1,BLUE);
		LedSetup(2,BLUE);
		LedSetup(3,BLUE);

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

		LedSetup(1,RED);
		LedSetup(2,BLUE);
		LedSetup(3,BLUE);

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;	

		LedSetup(1,RED);
		LedSetup(2,RED);
		LedSetup(3,BLUE);

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

		LedSetup(1,RED);
		LedSetup(2,RED);
		LedSetup(3,RED);

		if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
		
	}
	return;
}

/******************************************************************************

Name:         void Chaser2(void)

Description:

Input:        none

Output:       none

Misc:					#10
******************************************************************************/
void Chaser2(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{
	int i;

	VariableDelay = DelayTick;

	while(TRUE)
	{
		WDR();
		
		for (i=0;i<10;i++)
		{
			LedSetup(1,GREEN);
			LedSetup(2,RED);
			LedSetup(3,RED);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(1,RED);
			LedSetup(2,GREEN);
			LedSetup(3,RED);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(1,RED);
			LedSetup(2,RED);
			LedSetup(3,GREEN);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
		}

		for (i=0;i<10;i++)
		{
			LedSetup(1,BLUE);
			LedSetup(2,GREEN);
			LedSetup(3,GREEN);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;


			LedSetup(1,GREEN);
			LedSetup(2,BLUE);
			LedSetup(3,GREEN);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(1,GREEN);
			LedSetup(2,GREEN);
			LedSetup(3,BLUE);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY)  return;

		}

		for (i=0;i<10;i++)
		{
			LedSetup(1,RED);
			LedSetup(2,BLUE);
			LedSetup(3,BLUE);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(1,BLUE);
			LedSetup(2,RED);
			LedSetup(3,BLUE);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(1,BLUE);
			LedSetup(2,BLUE);
			LedSetup(3,RED);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
		}
	}
	return;
}

/******************************************************************************

Name:         void Chaser3(void)

Description:

Input:        none

Output:       none

Misc:					#11
******************************************************************************/
void Chaser3(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{
	int i;

	VariableDelay = DelayTick;

	while(TRUE)
	{
		WDR();
		
		for (i=0;i<10;i++)
		{
			LedSetup(3,OFF);
			LedSetup(1,GREEN);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(1,OFF);
			LedSetup(2,GREEN);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
            
			LedSetup(2,OFF);
			LedSetup(3,GREEN);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 
		}

		for (i=0;i<10;i++)
		{
			LedSetup(3,OFF);
			LedSetup(1,BLUE);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 

			LedSetup(1,OFF);
			LedSetup(2,BLUE);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(2,OFF);
			LedSetup(3,BLUE);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
		}

		for (i=0;i<10;i++)
		{
			LedSetup(3,OFF);
			LedSetup(1,RED);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(1,OFF);
			LedSetup(2,RED);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(2,OFF);
			LedSetup(3,RED);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
		}
	}
	return;
}

/******************************************************************************

Name:         void Chaser4(void)

Description:

Input:        none

Output:       none

Misc:					#12
******************************************************************************/
void Chaser4(ulong DelayTick, ulong DelayStep, ulong DelayMin)
{
	int i;

	VariableDelay = DelayTick;

	while(TRUE)
	{
		WDR();
		
		for (i=0;i<10;i++)
		{
			LedSetup(1,RED);
			LedSetup(2,OFF);
			LedSetup(3,OFF);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 

			LedSetup(1,OFF);
			LedSetup(2,RED);
			LedSetup(3,OFF);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(1,OFF);
			LedSetup(2,OFF);
			LedSetup(3,RED);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
			
			LedSetup(1,OFF);
			LedSetup(2,RED);
			LedSetup(3,OFF);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 
		}

		for (i=0;i<10;i++)
		{
			LedSetup(1,GREEN);
			LedSetup(2,OFF);
			LedSetup(3,OFF);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(1,OFF);
			LedSetup(2,GREEN);
			LedSetup(3,OFF);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 

			LedSetup(1,OFF);
			LedSetup(2,OFF);
			LedSetup(3,GREEN);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
			
			LedSetup(1,OFF);
			LedSetup(2,GREEN);
			LedSetup(3,OFF);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
		}

		for (i=0;i<10;i++)
		{
			LedSetup(1,BLUE);
			LedSetup(2,OFF);
			LedSetup(3,OFF);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 

			LedSetup(1,OFF);
			LedSetup(2,BLUE);
			LedSetup(3,OFF);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;

			LedSetup(1,OFF);
			LedSetup(2,OFF);
			LedSetup(3,BLUE);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return;
			
			LedSetup(1,OFF);
			LedSetup(2,BLUE);
			LedSetup(3,OFF);

			if (Delay(DelayStep, DelayMin) != REMOTE_NO_KEY) return; 
		}
	}
	return;
}

/******************************************************************************

Name:         void LedSetup(ushort Led,ushort Color)

Description:

Input:        none

Output:       none

Misc:					
******************************************************************************/
void LedSetup(ushort Led,ushort Color)
{
	if ((Led == 1) && (Color == OFF))   {Led1Red = 0; Led1Green = 0; Led1Blue = 0;}
	if ((Led == 1) && (Color == RED))   {Led1Red = Brightness; Led1Green = 0; Led1Blue = 0;}
	if ((Led == 1) && (Color == GREEN)) {Led1Red = 0; Led1Green = Brightness; Led1Blue = 0;}
	if ((Led == 1) && (Color == BLUE))  {Led1Red = 0; Led1Green = 0; Led1Blue = Brightness;}
	
	if ((Led == 2) && (Color == OFF))   {Led2Red = 0; Led2Green = 0; Led2Blue = 0;}
	if ((Led == 2) && (Color == RED))   {Led2Red = Brightness; Led2Green = 0; Led2Blue = 0;}
	if ((Led == 2) && (Color == GREEN)) {Led2Red = 0; Led2Green = Brightness; Led2Blue = 0;}
	if ((Led == 2) && (Color == BLUE))  {Led2Red = 0; Led2Green = 0; Led2Blue = Brightness;}
	
	if ((Led == 3) && (Color == OFF))   {Led3Red = 0; Led3Green = 0; Led3Blue = 0;}
	if ((Led == 3) && (Color == RED))   {Led3Red = Brightness; Led3Green = 0; Led3Blue = 0;}
	if ((Led == 3) && (Color == GREEN)) {Led3Red = 0; Led3Green = Brightness; Led3Blue = 0;}
	if ((Led == 3) && (Color == BLUE))  {Led3Red = 0; Led3Green = 0; Led3Blue = Brightness;}			
}

/******************************************************************************

name:         void delay(void)

description:

input:        none

output:       none

misc:					
**********************************************************************>********/
ushort Delay(ulong DelayStep, ulong DelayMin)
{
	long i;
	int j,k;
	ushort Key;

	RemoteKey = REMOTE_NO_KEY;
    
	for (i=0;i<VariableDelay + ((255-Brightness) * 8);i++)
	{
		WDR();

		if (RemoteKey != REMOTE_NO_KEY)
		{
			if (RemoteKey == REMOTE_UP)
			{
				VariableDelay -= DelayStep;
				if (VariableDelay <= DelayMin) VariableDelay = DelayMin;	
 				RemoteKey = REMOTE_NO_KEY;
			}

			else if (RemoteKey == REMOTE_DOWN)
			{
				VariableDelay += DelayStep;
				if (VariableDelay > 5000000) VariableDelay = 5000000;
				RemoteKey = REMOTE_NO_KEY;
			}
		    else if ((RemoteKey == REMOTE_LEFT) || (RemoteKey == REMOTE_RIGHT))
            {
            	RemoteKey = REMOTE_NO_KEY;
            }
			else
			{
			 	Key = RemoteKey;
			    RemoteKey = REMOTE_NO_KEY;
				return Key;
			}
		}
	}
	return REMOTE_NO_KEY;
}

/******************************************************************************

Name:         ushort GetRand(ushort Mask)

Description:  Pseudo random generator give value from 0 to 255

Input:        none

Output:       ushort

Misc:
******************************************************************************/
ushort GetRand(ushort Mask)
{
	static ushort x;
	uint y;
    
	x = x + (ADC * ADC) + (ADC + (x<<3));
    y = ((x & 0xff) * Mask) / 255;
    return (ushort)y;
}

/******************************************************************************

Name:         void LedInit(void)

Description:  This function initialize the Led driving

Input:        none

Output:       none

Misc:
******************************************************************************/
void LedInit(void)
{
  LED1_DDR |= LED1_RED + LED1_GREEN + LED1_BLUE;
  LED2_DDR |= LED2_RED + LED2_GREEN + LED2_BLUE;
  LED3_DDR |= LED3_RED + LED3_GREEN + LED3_BLUE;

  //Timer2
  TCCR2B = (1<<CS20);
  TIMSK2 |= (1<<TOIE2);	
}

/******************************************************************************

Name:         void LedEngine(void)

Description:  This function is the Led driver

Input:        none

Output:       none

Misc:

******************************************************************************/
#pragma interrupt_handler LedEngine:iv_TIMER2_OVF
void LedEngine(void)
{
	static ushort Ticker;

	Ticker++;
	if (Ticker == 255) Ticker = 0;

	//--------------------------
	
	if (Led1Red > Ticker)   LED1_PORT |= LED1_RED;
	else LED1_PORT &= ~LED1_RED;
	
	if (Led1Green  > Ticker) LED1_PORT |= LED1_GREEN;
	else LED1_PORT &= ~LED1_GREEN;
	
	if (Led1Blue > Ticker)  LED1_PORT |= LED1_BLUE;
	else LED1_PORT &= ~LED1_BLUE;

	//--------------------------

	if (Led2Red > Ticker)   LED2_PORT |= LED2_RED;
	else LED2_PORT &= ~LED2_RED;
	
	if (Led2Green > Ticker) LED2_PORT |= LED2_GREEN;
	else LED2_PORT &= ~LED2_GREEN;
	
	if (Led2Blue > Ticker)  LED2_PORT |= LED2_BLUE;
	else LED2_PORT &= ~LED2_BLUE;	
	
	//--------------------------
		
	if (Led3Red > Ticker)   LED3_PORT |= LED3_RED;
	else LED3_PORT &= ~LED3_RED;
	
	if (Led3Green > Ticker) LED3_PORT |= LED3_GREEN;
	else LED3_PORT &= ~LED3_GREEN;
	
	if (Led3Blue > Ticker)  LED3_PORT |= LED3_BLUE;
	else LED3_PORT &= ~LED3_BLUE;	
}

/******************************************************************************

Name:         void IRInit(void)

Description:  Init device for IR functions

Input:        none

Output:       none

Misc:         Share Timer 1 with Triac functions

******************************************************************************/
void IRInit(void)
{
  IR_DDR &= ~IR_INPUT;          // ICP as input on PB0
  IR_PORT |= IR_INPUT;          // Pull up on ICP pin
  TCCR1B |= (1<<CS11);          // Timer1 / 8
  TCCR1B |= (1<<ICES1);         // Rising edge on ICP
  TIMSK1 |= (1<<ICIE1);         // Int on ICP
}

/******************************************************************************

Name:         void IRDecode(void);

Description:  This routine is called whenever a rising edge (beginning
              of valid IR signal) is received.

Input:        none

Output:       Global variable LatchedIrData

Misc:         Sony VCR protocol

******************************************************************************/
#pragma interrupt_handler IRDecode:iv_TIMER1_CAPT
void IRDecode(void)
{
  static uint LastCapture;
  uint PulseWidth;
  static ushort IrPulseCount;
  static uint IrData;
	
  PulseWidth = ICR1 - LastCapture;
  LastCapture = ICR1;
 
  if (PulseWidth > (XTAL/4000))
  {
    IrPulseCount = 0;
    IrData = 0;
  }
  else
  {
    IrData = IrData >> 1;
    if (PulseWidth > (XTAL/5714)) IrData = IrData | 0x8000;
    IrPulseCount++;
    if (IrPulseCount == 12)
    {
      LatchedIrData = IrData >> 4;
      switch(LatchedIrData)
      {
        case 2176 : RemoteKey = 1;
        break;
        case 2177 : RemoteKey = 2;
        break;
        case 2178 : RemoteKey = 3;
        break;
        case 2179 : RemoteKey = 4;
        break;
        case 2180 : RemoteKey = 5;
        break;
        case 2181 : RemoteKey = 6;
        break;
        case 2182 : RemoteKey = 7;
        break;
        case 2183 : RemoteKey = 8;
        break;
        case 2184 : RemoteKey = 9;
        break;
        case 2208 : RemoteKey = 0;
        break;
        case 2224 : RemoteKey = REMOTE_REW;
        break;
        case 2227 : RemoteKey = REMOTE_REW;
        break;
        case 2225 : RemoteKey = REMOTE_FFWD;
        break;
        case 2228 : RemoteKey = REMOTE_FFWD;
        break;
        case 2231 : RemoteKey = REMOTE_UP;
        break;
        case 2238 : RemoteKey = REMOTE_UP;
        break;
        case 2230 : RemoteKey = REMOTE_DOWN;
        break;
        case 2237 : RemoteKey = REMOTE_DOWN;
        break;
        case 2235 : RemoteKey = REMOTE_RIGHT;
        break;
        case 2194 : RemoteKey = REMOTE_RIGHT;
        break;
        case 2234 : RemoteKey = REMOTE_LEFT;
        break;
        case 2195 : RemoteKey = REMOTE_LEFT;
        break;
        case 2233 : RemoteKey = REMOTE_PAUSE;
        break;
        case 2232 : RemoteKey = REMOTE_STOP;
        break;
        case 2226 : RemoteKey = REMOTE_PLAY;
        break;
        case 2271 : RemoteKey = REMOTE_MUTE;
        break;
        case 2262 : RemoteKey = REMOTE_MENU;
        break;
        case 2197 : RemoteKey = REMOTE_POWER;
        break;
        default   : RemoteKey = -1;
        break;
      }
    }
  }
  if ((RemoteKey == REMOTE_LEFT) && (Brightness > 0)) Brightness--;
  if ((RemoteKey == REMOTE_RIGHT) && (Brightness < 255)) Brightness++;
}

/******************************************************************************

Name:         void ADInit(void)

Description:  Init AD for random return value from external noise

Input:        none

Output:       none

Misc:         

******************************************************************************/
void ADInit(void)
{
	ADMUX = (1<<REFS1) + (1<<REFS0) + 0x0b;
	ADCSRA = (1<<ADEN) + (1<<ADATE) + (1<<ADPS1) + (1<<ADPS0);
 	ADCSRA |= (1<<ADSC);
}
